@@ -1,8 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2007
- * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
  * This file is part of SIP.
  * 
@@ -88,6 +87,8 @@
 #define CLASS_QOBJECT_SUB   0x00800000  /* It is derived from QObject. */
 #define CLASS_DTOR_HOLD_GIL 0x01000000  /* The dtor holds the GIL. */
 #define CLASS_QT_META_TYPE  0x02000000  /* Register as a Qt meta type. */
+#define CLASS_NO_QMETAOBJECT    0x04000000  /* It has no QMetaObject. */
+#define CLASS_CAN_ASSIGN    0x08000000  /* It has an assignment operator. */
 
 #define hasSigSlots(cd)     ((cd)->classflags & CLASS_HAS_SIGSLOTS)
 #define setHasSigSlots(cd)  ((cd)->classflags |= CLASS_HAS_SIGSLOTS)
@@ -127,6 +128,10 @@
 #define setIsHoldGILDtor(c) ((cd)->classflags |= CLASS_DTOR_HOLD_GIL)
 #define registerQtMetaType(c)   ((cd)->classflags & CLASS_QT_META_TYPE)
 #define setRegisterQtMetaType(c)    ((cd)->classflags |= CLASS_QT_META_TYPE)
+#define noQMetaObject(c)    ((cd)->classflags & CLASS_NO_QMETAOBJECT)
+#define setNoQMetaObject(c) ((cd)->classflags |= CLASS_NO_QMETAOBJECT)
+#define canAssign(c)        ((cd)->classflags & CLASS_CAN_ASSIGN)
+#define setCanAssign(c)     ((cd)->classflags |= CLASS_CAN_ASSIGN)
 
 #define isPublicDtor(cd)    ((cd)->classflags & SECT_IS_PUBLIC)
 #define setIsPublicDtor(cd) ((cd)->classflags |= SECT_IS_PUBLIC)
@@ -142,6 +147,7 @@
 #define CTOR_EXPLICIT       0x00000200  /* The ctor is explicit. */
 #define CTOR_CAST           0x00000400  /* The ctor is a cast. */
 #define CTOR_HOLD_GIL       0x00000800  /* The ctor holds the GIL. */
+#define CTOR_XFERRED        0x00001000  /* Ownership is transferred. */
 
 #define isPublicCtor(c)     ((c)->ctorflags & SECT_IS_PUBLIC)
 #define setIsPublicCtor(c)  ((c)->ctorflags |= SECT_IS_PUBLIC)
@@ -157,14 +163,19 @@
 #define isCastCtor(c)       ((c)->ctorflags & CTOR_CAST)
 #define isHoldGILCtor(c)    ((c)->ctorflags & CTOR_HOLD_GIL)
 #define setIsHoldGILCtor(c) ((c)->ctorflags |= CTOR_HOLD_GIL)
+#define isResultTransferredCtor(c)  ((c)->ctorflags & CTOR_XFERRED)
+#define setIsResultTransferredCtor(c)   ((c)->ctorflags |= CTOR_XFERRED)
 
 
 /* Handle member flags. */
 
 #define MEMBR_NUMERIC       0x0001      /* It is a numeric slot. */
+#define MEMBR_NO_ARG_PARSER 0x0002      /* Don't generate an argument parser. */
 
 #define isNumeric(m)        ((m)->memberflags & MEMBR_NUMERIC)
 #define setIsNumeric(m)     ((m)->memberflags |= MEMBR_NUMERIC)
+#define noArgParser(m)      ((m)->memberflags & MEMBR_NO_ARG_PARSER)
+#define setNoArgParser(m)   ((m)->memberflags |= MEMBR_NO_ARG_PARSER)
 
 
 /* Handle enum flags.  These are combined with the section flags. */
@@ -286,6 +297,7 @@
 #define ARG_OUT             0x0400  /* It returns a result. */
 #define ARG_CONSTRAINED     0x0800  /* Suppress type conversion. */
 #define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
+#define ARG_RESULT_SIZE     0x2000  /* It defines the result size. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
 #define setIsReference(a)   ((a)-> argflags |= ARG_IS_REF)
@@ -315,6 +327,8 @@
 #define setIsConstrained(a) ((a)->argflags |= ARG_CONSTRAINED)
 #define resetIsConstrained(a)   ((a)->argflags &= ~ARG_CONSTRAINED)
 #define isSingleShot(a)     ((a)->argflags & ARG_SINGLE_SHOT)
+#define isResultSize(a)     ((a)->argflags & ARG_RESULT_SIZE)
+#define setResultSize(a)    ((a)->argflags |= ARG_RESULT_SIZE)
 
 
 /* Handle name flags. */
@@ -342,6 +356,14 @@
 #define setIsTransferVH(vh) ((vh)->vhflags |= VH_TRANSFERS)
 
 
+/* Handle mapped type flags. */
+
+#define MT_NO_RELEASE       0x01    /* Do not generate a release function. */
+
+#define noRelease(mt)       ((mt)->mtflags & MT_NO_RELEASE)
+#define setNoRelease(mt)    ((mt)->mtflags |= MT_NO_RELEASE)
+
+
 /* Slot types. */
 
 typedef enum {
@@ -447,7 +469,8 @@ typedef enum {
     cbool_type,
     sstring_type,
     wstring_type,
-    fake_void_type
+    fake_void_type,
+    ssize_type
 } argType;
 
 
@@ -540,7 +563,7 @@ typedef struct _nodeDef {
 
 typedef struct _codeBlock {
     char *frag;                         /* The code itself. */
-    char *filename;                     /* The original file. */
+    const char *filename;               /* The original file. */
     int linenr;                         /* The line in the file. */
     struct _codeBlock *next;            /* Next in the list. */
 } codeBlock;
@@ -549,8 +572,8 @@ typedef struct _codeBlock {
 /* A module definition. */
 
 typedef struct _moduleDef {
-    char *fullname;                     /* The full module name. */
-    char *name;                         /* The module base name. */
+    const char *fullname;               /* The full module name. */
+    const char *name;                   /* The module base name. */
     int version;                        /* The module version. */
     int modflags;                       /* The module flags. */
     int qobjclass;                      /* QObject class, -1 if none. */
@@ -694,6 +717,7 @@ typedef struct _ifaceFileList {
 /* A mapped type. */
 
 typedef struct _mappedTypeDef {
+    int mtflags;                        /* The mapped type flags. */
     argDef type;                        /* The type being mapped. */
     int mappednr;                       /* The mapped type number. */
     ifaceFileDef *iff;                  /* The interface file. */
@@ -877,7 +901,7 @@ typedef struct _classDef {
     int classflags;                     /* The class flags. */
     int userflags;                      /* The user type flags. */
     int classnr;                        /* The class number. */
-    char *pyname;                       /* The Python name. */
+    const char *pyname;                 /* The Python name. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _classDef *ecd;              /* The enclosing scope. */
     struct _classDef *real;             /* The real class if this is a proxy or extender. */
@@ -985,13 +1009,14 @@ void fatal(char *,...);
 void fatalScopedName(scopedNameDef *);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
 void *sipMalloc(size_t);
-char *sipStrdup(char *);
+char *sipStrdup(const char *);
 char *concat(const char *, ...);
-void append(char **,char *);
-ifaceFileList *addToUsedList(ifaceFileList **, ifaceFileDef *);
+void append(char **, const char *);
+void addToUsedList(ifaceFileList **, ifaceFileDef *);
 int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
-int sameTemplateSignature(signatureDef *sd1, signatureDef *sd2, int deep);
+int sameTemplateSignature(signatureDef *tmpl_sd, signatureDef *args_sd,
+        int deep);
 int sameScopedName(scopedNameDef *,scopedNameDef *);
 int sameBaseType(argDef *,argDef *);
 char *scopedNameTail(scopedNameDef *);
@@ -1019,6 +1044,7 @@ ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod, scopedNameDef *fqname,
 int optNoEmitters(sipSpec *pt);
 int optRegisterTypes(sipSpec *pt);
 int optQ_OBJECT4(sipSpec *pt);
+int optAssignmentHelpers(sipSpec *pt);
 void yywarning(char *);
 nameDef *cacheName(sipSpec *pt, const char *name);
 
