@@ -1,8 +1,7 @@
 /*
  * The parse tree transformation module for SIP.
  *
- * Copyright (c) 2007
- * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
  * This file is part of SIP.
  * 
@@ -69,8 +68,8 @@ static void searchEnums(sipSpec *,scopedNameDef *,argDef *);
 static void searchClasses(sipSpec *,moduleDef *mod,scopedNameDef *,argDef *);
 static void appendToMRO(mroDef *,mroDef ***,classDef *);
 static void moveMainModuleCastsSlots(sipSpec *pt, moduleDef *mod);
-static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd);
-static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd);
+static void moveClassCasts(moduleDef *mod, classDef *cd);
+static void moveGlobalSlot(moduleDef *mod, memberDef *gmd);
 static void filterMainModuleVirtualHandlers(moduleDef *mod);
 static void filterModuleVirtualHandlers(moduleDef *mod);
 static ifaceFileDef *getIfaceFile(argDef *ad);
@@ -81,6 +80,7 @@ static void registerMetaType(classDef *cd);
 static void addComplementarySlots(sipSpec *pt, classDef *cd);
 static void addComplementarySlot(sipSpec *pt, classDef *cd, memberDef *md,
         slotType cslot, const char *cslot_name);
+static void resolveInstantiatedClassTemplate(sipSpec *pt, argDef *type);
 
 
 /*
@@ -94,7 +94,6 @@ void transform(sipSpec *pt)
     classList *newl;
     overDef *od;
     mappedTypeDef *mtd;
-    virtHandlerDef *vhd;
 
     /*
      * The class list has the main module's classes at the front and the
@@ -514,18 +513,18 @@ static void moveMainModuleCastsSlots(sipSpec *pt, moduleDef *mod)
 
     for (cd = pt->classes; cd != NULL; cd = cd->next)
         if (cd->iff->module == mod)
-            moveClassCasts(pt, mod, cd);
+            moveClassCasts(mod, cd);
 
     for (md = mod->othfuncs; md != NULL; md = md->next)
         if (md->slot != no_slot && md->module == mod)
-            moveGlobalSlot(pt, mod, md);
+            moveGlobalSlot(mod, md);
 }
 
 
 /*
  * Move any class casts to its correct class, or publish as a ctor extender.
  */
-static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
+static void moveClassCasts(moduleDef *mod, classDef *cd)
 {
     argList *al;
 
@@ -588,7 +587,7 @@ static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
 /*
  * If possible, move a global slot to its correct class.
  */
-static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
+static void moveGlobalSlot(moduleDef *mod, memberDef *gmd)
 {
     overDef **odp = &mod->overs, *od;
 
@@ -1302,6 +1301,10 @@ static void addDefaultCopyCtor(classDef *cd)
  
         *tailp = copyct;
     }
+
+    /* We assume it has an assignment operator if it has a public copy ctor. */
+    if (isPublicCtor(copyct))
+        setCanAssign(cd);
 }
 
 
@@ -1320,8 +1323,8 @@ static void transformScopeOverloads(sipSpec *pt, classDef *scope,
         resolveFuncTypes(pt, od->common->module, scope, od);
 
         /*
-         * Now check that the Python signature doesn't conflict with an
-         * earlier one.
+         * Now check that the Python signature doesn't conflict with an earlier
+         * one.
          */
         for (prev = overs; prev != od; prev = prev->next)
         {
@@ -1831,12 +1834,8 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     else
         ifaceFileIsUsed(&vd->module->used, vtype);
 
-    /*
-     * Instance variables or static class variables (unless they are constants)
-     * need a handler.
-     */
-    if (vd->ecd != NULL && vd->accessfunc == NULL &&
-        (!isStaticVar(vd) || vtype->nrderefs != 0 || !isConstArg(vtype)))
+    /* Scoped variables need a handler unless they have %AccessCode. */
+    if (vd->ecd != NULL && vd->accessfunc == NULL)
     {
         setNeedsHandler(vd);
         setHasVarHandlers(vd->ecd);
@@ -2056,10 +2055,12 @@ static void defaultInput(argDef *ad)
 static void defaultOutput(argDef *ad)
 {
     if (!isOutArg(ad) && !isInArg(ad))
+    {
         if (isConstArg(ad))
             setIsInArg(ad);
         else
             setIsOutArg(ad);
+    }
 }
 
 
@@ -2239,13 +2240,24 @@ static int sameArgType(argDef *a1, argDef *a2, int strict)
 /*
  * Compare two basic types and return TRUE if they are the same.
  */
-
-int sameBaseType(argDef *a1,argDef *a2)
+int sameBaseType(argDef *a1, argDef *a2)
 {
     /* The types must be the same. */
+    if (a1->atype != a2->atype)
+    {
+        /*
+         * If we are comparing a template with those that have already been
+         * used to instantiate a class then we need to compare with the class
+         * name.  Hopefully this won't have wider side effects.
+         */
+        if (a1->atype == class_type && a2->atype == defined_type)
+            return sameScopedName(a1->u.cd->iff->fqcname, a2->u.snd);
+
+        if (a1->atype == defined_type && a2->atype == class_type)
+            return sameScopedName(a1->u.snd, a2->u.cd->iff->fqcname);
 
-    if (a1 -> atype != a2 ->atype)
         return FALSE;
+    }
 
     switch (a1 -> atype)
     {
@@ -2523,21 +2535,7 @@ static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *defscope, argDef
     }
 
     /* See if the type refers to an instantiated template. */
-    if (type->atype == template_type)
-    {
-        classDef *cd;
-
-        for (cd = pt->classes; cd != NULL; cd = cd->next)
-            if (cd->td != NULL &&
-                sameScopedName(cd->td->fqname, type->u.td->fqname) &&
-                sameSignature(&cd->td->types, &type->u.td->types, TRUE))
-            {
-                type->atype = class_type;
-                type->u.cd = cd;
-
-                break;
-            }
-    }
+    resolveInstantiatedClassTemplate(pt, type);
 
     /* Replace the base type if it has been mapped. */
     if (type -> atype == struct_type || type -> atype == template_type)
@@ -2545,15 +2543,15 @@ static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *defscope, argDef
         searchMappedTypes(pt,NULL,type);
 
         /*
-         * If we still have a template then see if we need to
-         * automatically instantiate it.
+         * If we still have a template then see if we need to automatically
+         * instantiate it.
          */
         if (type->atype == template_type)
         {
             mappedTypeTmplDef *mtt;
 
             for (mtt = pt->mappedtypetemplates; mtt != NULL; mtt = mtt->next)
-                if (sameScopedName(type->u.td->fqname, mtt->mt->type.u.td->fqname) && sameTemplateSignature(&type->u.td->types, &mtt->mt->type.u.td->types, TRUE))
+                if (sameScopedName(type->u.td->fqname, mtt->mt->type.u.td->fqname) && sameTemplateSignature(&mtt->mt->type.u.td->types, &type->u.td->types, TRUE))
                 {
                     type->u.mtd = instantiateMappedTypeTemplate(pt, mod, mtt, type);
                     type->atype = mapped_type;
@@ -2565,6 +2563,38 @@ static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *defscope, argDef
 }
 
 
+/*
+ * If the type corresponds to a previously instantiated class template then
+ * replace it with the class that was created.
+ */
+static void resolveInstantiatedClassTemplate(sipSpec *pt, argDef *type)
+{
+    int a;
+    classDef *cd;
+    templateDef *td;
+    signatureDef *sd;
+
+    if (type->atype != template_type)
+        return;
+
+    td = type->u.td;
+    sd = &td->types;
+
+    for (a = 0; a < sd->nrArgs; ++a)
+        resolveInstantiatedClassTemplate(pt, &sd->args[a]);
+
+    for (cd = pt->classes; cd != NULL; cd = cd->next)
+        if (cd->td != NULL && sameScopedName(cd->td->fqname, td->fqname) &&
+            sameSignature(&cd->td->types, sd, TRUE))
+        {
+            type->atype = class_type;
+            type->u.cd = cd;
+
+            break;
+        }
+}
+
+
 /*
  * Instantiate a mapped type template and return it.
  */
@@ -2798,7 +2828,7 @@ static void ifaceFilesAreUsedByOverload(ifaceFileList **used, overDef *od)
         int a;
 
         for (a = 0; a < ta->nrArgs; ++a)
-            ifaceFileIsUsed(used, &ta->args[a]);
+            addToUsedList(used, ta->args[a]->cd->iff);
     }
 }
 
@@ -2811,8 +2841,7 @@ static void ifaceFileIsUsed(ifaceFileList **used, argDef *ad)
 {
     ifaceFileDef *iff;
 
-    /* Make sure we don't try to add an interface file to its own list. */
-    if ((iff = getIfaceFile(ad)) != NULL && &iff->used != used)
+    if ((iff = getIfaceFile(ad)) != NULL)
     {
         addToUsedList(used, iff);
 
@@ -2826,8 +2855,7 @@ static void ifaceFileIsUsed(ifaceFileList **used, argDef *ad)
             ifaceFileList *iffl = iff->used;
 
             for (iffl = iff->used; iffl != NULL; iffl = iffl->next)
-                if (&iffl->iff->used != used)
-                    addToUsedList(used, iffl->iff);
+                addToUsedList(used, iffl->iff);
         }
     }
 }
