@@ -1,8 +1,7 @@
 /*
  * This file defines the SIP library internal interfaces.
  *
- * Copyright (c) 2007
- * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
  * This file is part of SIP.
  * 
@@ -81,13 +80,14 @@ PyObject *sip_api_convert_from_instance(void *cppPtr, sipWrapperType *type,
 void sip_api_common_dtor(sipWrapper *sipSelf);
 void sip_api_start_thread(void);
 void sip_api_end_thread(void);
-PyObject *sip_api_convert_from_void_ptr(void *val);
 PyObject *sip_api_convert_from_named_enum(int eval, PyTypeObject *et);
 int sip_api_wrapper_check(PyObject *o);
 void sip_api_free_connection(sipSlotConnection *conn);
-int sip_api_emit_to_slot(sipSlot *slot, PyObject *sigargs);
+int sip_api_emit_to_slot(const sipSlot *slot, PyObject *sigargs);
 int sip_api_same_connection(sipSlotConnection *conn, void *tx, const char *sig,
         PyObject *rxObj, const char *slot);
+PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs);
+void sip_api_parse_type(const char *type, sipSigArg *arg);
 
 
 /*
